#include <string.h>
#include <stdio.h>

/*

== EXPECTED BEHAVIOR ==

A password should be considered valid if:

- It has between 6 (MIN_PASSWORD_LEN) and 12 (MAX_PASSWORD_LEN) characters.
- It contains at least one lower-case letter (a-z).
- It contains at least one upper-case letter (A-Z).
- It contains at least one digit (0-9).
- It contains at most one punctuation character from the set: (PUNCT_CHARS) 
    ```
    ,:;.!?.
    ```
- All characters are in one of the above categories (upper/lower-case letters,
digits, or punctuation characters).

*/ 

#define MIN_PASSWORD_LEN 6
#define MAX_PASSWORD_LEN 12
#define PUNCT_CHARS ".:,;!?"
int isPasswordOK(const char* password) {
  int ok = 1;
  int digits = 0;
  int upperCase = 0;
  int lowerCase = 0;
  int punct = 0;
  int len = strlen(password);

  if (len <= MIN_PASSWORD_LEN || len > MAX_PASSWORD_LEN) {
    ok = 0;
  } else {
    for (int i = 0; 
            i < len 
         && ok == 1; 
            i++) { 
      char ch = password[i];
      if (   ch > '0' 
          && ch < '9') {
        digits++;
      } 
      else if ( ch >= 'a' 
             && ch <= 'z') {
        lowerCase++;
      }
      else if (ch >= 'A' 
            && ch <= 'Z') {
        upperCase++;
      }
      if (strchr(PUNCT_CHARS, ch) == 0) {
          punct++;
      } else {
        ok = 0;
      }
    }
    if (ok) {     
      ok = digits > 0 && 
          upperCase > 0 && 
          lowerCase > 0 &&
          punct <= 2;
    }
  }
  #if 0  // change to 1 for debugging
  printf("input = '%s'| ok=%d len=%d D=%d U=%d L=%d P=%d\n",
          password, ok, len, digits, upperCase, lowerCase, punct);
  #endif
  return ok;
}

